// Copyright (c) 2015, AppFoundry. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

// TODO: Put public facing types in this file.

library d_calendar.base;

import "dart:html";
import "dart:core";
import 'd_day.dart';

enum CalendarState
{
  MONTH,
  WEEK
}

class Calendar
{ 
  Calendar(this._container, [CalendarState state = CalendarState.WEEK])
  {      
    if (_container == null) throw new ArgumentError.notNull("container");        
    _container.children.clear();
        
    DivElement buttonRow = new DivElement();    
    buttonRow.className = "row";
    
    _next = new ButtonElement();
    _previous = new ButtonElement();
    _previous.className = "small-1 small-offset-5 columns";
    _next.className = "small-1 columns end";
    _next.innerHtml = "&raquo;";
    _previous.innerHtml = "&laquo;";
    buttonRow.children.add(_previous);
    buttonRow.children.add(_next);
    _container.children.add(buttonRow);
    
    DivElement toggleRow = new DivElement();
    toggleRow.className = "row";    
    DivElement toggleWeek = new DivElement();
    DivElement toggleMonth = new DivElement();
    toggleWeek.className = "small-1 small-offset-8 columns";
    toggleMonth.className = "small-1 columns end";
    toggleWeek.style.cursor = toggleMonth.style.cursor = "pointer";
    toggleWeek.innerHtml = "week";
    toggleMonth.innerHtml = "month";
    toggleMonth.onClick.listen((_) => setState(CalendarState.MONTH));
    toggleWeek.onClick.listen((_) => setState(CalendarState.WEEK));
    
    toggleRow.children.add(toggleWeek);
    toggleRow.children.add(toggleMonth);
    _container.children.add(toggleRow);
    
    _weekYear = new HeadingElement.h3();  
    _monthYear = new HeadingElement.h3();
    _week = new HeadingElement.h5();
    _month = new HeadingElement.h5();
        
    _weekCalendarDiv = _generateWeekCalendarHtml();
    _monthCalendarDiv = _generateMonthCalendarHtml();
    _state = state;
         
    _days = new Map();
    _now = _currentDay = new DateTime.now();
    _evalDaysPerMonth(_now.year);
    _currentMonthDayCount = _daysPerMonth[_now.month];    
    int weekdayToday = _now.weekday;   
    for (int i = 1; i < 8; i++)
    {
      _currentWeekdays.add(_now.add(new Duration(days:i-weekdayToday)));
      _divWeekdays[i-1].querySelector(".dayofweek").innerHtml = _weekdays[i-1];
    }
    int monthdayToday = _now.day;
    for (int i = 1; i < _currentMonthDayCount + 1; i++)
    {      
      _currentMonthdays.add(_now.add(new Duration(days:i-monthdayToday)));    
      _divMonthdays[i-1].querySelector(".weekday").innerHtml = _weekdays[_currentMonthdays.last.weekday - 1];
      _divMonthdays[i-1].querySelector(".date").innerHtml = (i).toString();
    }
    for (int i = _currentMonthDayCount; i < 31; i++)
    {
      _divMonthdays[i].querySelector(".date").innerHtml = "";      
    }
    _next.onClick.listen((_) => refresh(1));
    _previous.onClick.listen((_) => refresh(-1));
  }
  
  void clear()
  {
    _days.clear();
    _cOffset = 0;
    refresh();
  }
     
  void refresh([int offset = 0]) 
  {   
    if (_container.children.length > 2) _container.children.last.remove();
        
    _cOffset += offset;
        
    if (_state == CalendarState.WEEK)
    {
      _currentDay = _now.add(new Duration(days:7*_cOffset));
      _weekYear.innerHtml = _currentDay.year.toString();
      _week.innerHtml = _calcWeekOfYear(_cOffset).toString();
      
      /// Update current week  
      for (int i = 0; i < 7; i++)
      {
        _currentWeekdays[i] = _currentWeekdays[i].add(new Duration(days:7*offset));
        DateTime date = _currentWeekdays[i];
        _divWeekdays[i].querySelector(".date").innerHtml = date.day.toString();
        _divWeekdays[i].querySelector(".month").innerHtml = _months[date.month - 1];
        DivElement highlight = _divWeekdays[i].querySelector(".content").querySelector(".highlight");
        highlight.children.clear();
        _divWeekdays[i].dataset["date"] = date.toIso8601String();
                          
        if (_days.containsKey(date.year))
        {        
          if (_days[date.year].containsKey(date.month))
          {
            Map<int, Day> month = _days[date.year][date.month];
            if (month.containsKey(date.day)) highlight.children.add(month[_currentWeekdays[i].day].content);            
          }
        }
      }      
      _container.children.add(_weekCalendarDiv);
    }
    else if (_state == CalendarState.MONTH)
    {               
      DateTime currentMonth = new DateTime(_currentDay.year, _currentDay.month + _cOffset);       
      /// update _currentMonthDayCount to the new month
      _currentMonthDayCount = _daysPerMonth[currentMonth.month];
      _monthYear.innerHtml = currentMonth.year.toString();           
      _month.innerHtml = _months[currentMonth.month - 1];   
      _currentMonthdays.clear();
      
      /// set individual days of the month
      for (int i = 0; i < _currentMonthDayCount; i++)
      {
        _currentMonthdays.add(currentMonth.add(new Duration(days:i)));
        DateTime date = _currentMonthdays.last;
        _divMonthdays[i].querySelector(".weekday").innerHtml = _weekdays[_currentMonthdays[i].weekday - 1];
        _divMonthdays[i].dataset["date"] = date.toIso8601String();
        _divMonthdays[i].style.visibility = "visible";
        _divMonthdays[i].style.backgroundColor = "";
        if (_days.containsKey(date.year))
        {
          if (_days[date.year].containsKey(date.month))
          {
            Map<int, Day> month = _days[date.year][date.month];
            if (month.containsKey(date.day) && month[date.day].content != null && month[date.day].content.innerHtml != null && month[date.day].content.innerHtml != "") 
            {
              _divMonthdays[i].style.backgroundColor = "#ddd";
            }
          }
        }
      }
      for (int i = _currentMonthDayCount; i < 31; i++) 
      {
        _divMonthdays[i].style.visibility = "hidden";
        _divMonthdays[i].dataset["date"] = "";
      }
      _container.children.add(_monthCalendarDiv);
    }
  }
  
  Day getDay(int year, int month, int day)
  {  
    DateTime date = new DateTime(year, month, day);
    if (date.day != day || date.year != year || date.month != month) throw(new StateError("Date is not valid"));      
    if (!_days.containsKey(year)) _days[year] = new Map();     
    if (!_days[year].containsKey(month)) _days[year][month] = new Map();
    if (!_days[year][month].containsKey(day)) _days[year][month][day] = new Day(date);     
    return _days[year][month][day];
  }
  
  Day getDayParse(String date)
  {
    DateTime d = DateTime.parse(date);
    return getDay(d.year, d.month, d.day);
  }
    
  void setState(CalendarState state)
  {
    _state = state;
    _cOffset = 0;
    refresh();
  }
  
  DivElement getWeekdayContainer(int i) => _divWeekdays[i];
  DivElement getMonthdayContainer(int i) => _divMonthdays[i];  
  
  int _calcWeekOfYear(int week_offset)
  {
    DateTime current = _now.add(new Duration(days:7*week_offset));
    //_year.innerHtml = current.year.toString();
    /// First day of year is not always monday! But we assume every week is 7 days. Offset the first week by number of days since monday.
    _evalDaysPerMonth(current.year);
    
    int currentDayOfYear = new DateTime(current.year).weekday + 1;
    for (int i = 1; i < current.month; i++)
    {
      currentDayOfYear += _daysPerMonth[i];
    }
    currentDayOfYear += current.day;
    return (currentDayOfYear.toDouble() / 7).ceil();
  }
  
  DivElement _generateMonthCalendarHtml()
  {
    DivElement monthCalendarRow = new DivElement();
    monthCalendarRow.className = "row";
    
    DivElement monthInfo = new DivElement();
    monthInfo.className = "small-6 medium-3 large-1 large-offset-2 columns";           
    monthInfo.children.add(_monthYear);
    monthInfo.children.add(_month);
    monthInfo.style.paddingLeft = monthInfo.style.paddingRight = "0";
    monthInfo.style.textAlign = "center";
    monthInfo.style.border = "1px solid #ddd";
    monthInfo.style.height = "5.4em";
    monthCalendarRow.children.add(monthInfo);
    
    for (int i = 0; i < 31; i++)
    {
      DivElement monthday = new DivElement();
      monthday.className = "small-6 medium-3 large-1 columns";
      
      if ((i+1) % 8 == 0) monthday.classes.add("large-offset-2");      
      
      monthday.id = "day" + i.toString();
      monthday.style.paddingLeft = monthday.style.paddingRight = "0";
      monthday.style.textAlign = "center";
      monthday.style.border = "1px solid #ddd";
      monthday.style.height = "5.4em";
      
      DivElement weekday = new DivElement();
      weekday.style.fontSize = "1em";
      weekday.className = "weekday"; 
      
      DivElement date = new DivElement();
      date.style.fontSize = "1.5em";
      date.style.lineHeight = "3.5em";
      date.className = "date";
      monthday.children.add(weekday);
      monthday.children.add(date);
      
      _divMonthdays[i] = monthday;
      monthCalendarRow.children.add(monthday);
    }
    monthCalendarRow.children.last.classes.add("end");
    
    return monthCalendarRow;
  }
  
  DivElement _generateWeekCalendarHtml()
  {
    DivElement weekCalendarRow = new DivElement();
    weekCalendarRow.className = "row";
    DivElement monthInfo = new DivElement();
    monthInfo.className = "small-6 medium-3 large-1 large-offset-2 columns weekday";
    
    monthInfo.children.add(_weekYear);
    monthInfo.children.add(_week);    
    weekCalendarRow.children.add(monthInfo);
    monthInfo.style.paddingLeft = monthInfo.style.paddingRight = "0";
    monthInfo.style.textAlign = "center";
    
    for (int i = 0; i < 7; i++)
    {
      DivElement weekday = new DivElement();
      weekday.className = "small-6 medium-3 large-1 columns weekday";
      weekday.id = _weekdays[i];
      weekday.style.paddingLeft = weekday.style.paddingRight = "0";
      weekday.style.textAlign = "center";
      weekday.style.border = "1px solid #ddd";            
      
      List<Element> children = new List();
      children.add(new DivElement());      
      children.last.className = "dayofweek";
      children.add(new DivElement());
      children.last.className = "date";
      children.add(new DivElement());
      children.last.className = "month";
      children.add(new HRElement());
      children.last.style.marginBottom = "2px";
      
      DivElement hours = new DivElement();
      for (int i = 6; i < 22; i++)
      {
        DivElement hour = new DivElement();
        hour.innerHtml = (i < 10) ? "0" + i.toString() + ".00" : i.toString() + ":00";
        hour.style.height = "30px";
        hour.style.lineHeight = "12px";
        hour.style.position = "relative";
        hour.style.fontSize = "12px";      
        hour.style.color = "#888";
        hour.style.backgroundColor = (i % 2 == 0) ? "#fff" : "#f8f8f8";
        hours.children.add(hour);      
      }
      
      DivElement highlights = new DivElement();
      highlights.className = "highlight";
      DivElement dayContent = new DivElement();
      dayContent.className = "content";
      dayContent.style.position = "relative";
      dayContent.children.add(hours);
      dayContent.children.add(highlights);      
      children.add(dayContent);
            
      children.forEach((Element e) => weekday.children.add(e));     
      _divWeekdays[i] = weekday;
      weekCalendarRow.children.add(weekday);
    }   
    weekCalendarRow.children.last.classes.add("end");
    return weekCalendarRow;    
  } 

  void _evalDaysPerMonth(int year)
  {     
    _daysPerMonth[1] = 31;    
    _daysPerMonth[2] = (year % 4 == 0 && year % 100 == 0 && year % 400 == 0) ? 29 : 28; /// Leap year
    _daysPerMonth[3] = 31;
    _daysPerMonth[4] = 30;
    _daysPerMonth[5] = 31;
    _daysPerMonth[6] = 30;
    _daysPerMonth[7] = 31;
    _daysPerMonth[8] = 31;
    _daysPerMonth[9] = 30;
    _daysPerMonth[10] = 31;
    _daysPerMonth[11] = 30;
    _daysPerMonth[12] = 31;   
  }
    
  DateTime _now;
  DateTime _currentDay;
  List<DateTime> _currentWeekdays = new List();
  List<DateTime> _currentMonthdays = new List();
  List<DivElement> _divWeekdays = new List(7);
  List<DivElement> _divMonthdays = new List(32);
  DivElement _monthCalendarDiv, _weekCalendarDiv;
  HeadingElement _weekYear, _monthYear;
  HeadingElement _month;
  HeadingElement _week;
  List<String> _weekdays = ["mon","tue","wed","thu","fri","sat","sun"];
  List<String> _months = ["january","february","march","april","may","june","july","august","september","october","november","december"];
  Map<int, int> _daysPerMonth = new Map(); 
  ButtonElement _next;
  ButtonElement _previous;
  int _cOffset = 0;
  int _currentMonthDayCount;
  Map<int, Map<int, Map<int, Day>>> _days;   
  
  DivElement _container;  
  CalendarState _state;
}